;===========================================================================================
; Rotbit.asm  Fasm Flat assembler
; Rotation de texte et bitmap
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,ClassName,Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU+\
                          WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,msg,NULL,0,0
    or eax,eax
    jz ExitProgram
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam
    local .DestDC:DWORD, .SrcDC:DWORD, .hBmp1:DWORD, .hBmp2:DWORD

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande de fin d'application
        ; ----------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE

        ; Dfinir une fonte de texte
        ; --------------------------
        invoke CreateFont,8,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                       CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                      MSSansSerif
        mov [hFontMSSS],eax                      ;fonte handle

        ; Construit tous les controles
        ; ----------------------------
        stdcall CreateControls,[@hWnd]

    .elseif [@uMsg] = WM_PAINT

        invoke GetClientRect,[@hWnd],rect1
        .if [rect1.right] & [rect1.bottom]
            invoke BeginPaint,[@hWnd],ps

            ; Cration d'un dispositif de contexte mmoire pour la destination
            ; ----------------------------------------------------------------
            invoke CreateCompatibleDC,[ps.hdc]
            mov [.DestDC],eax

            ; Cration du bitmap de destination
            ; ---------------------------------
            invoke CreateCompatibleBitmap,[ps.hdc],[rect1.right],[rect1.bottom]
            mov [.hBmp1],eax

            ; L'inclure dans le dispositif de contexte de destination
            ; -------------------------------------------------------
            invoke SelectObject,[.DestDC],[.hBmp1]
            push eax

            ; Fixer la couleur du fond de la destination
            ; ------------------------------------------
            invoke GetStockObject,WHITE_BRUSH
            invoke FillRect,[.DestDC],rect1,eax
    
            ; Cration d'un dispositif de contexte mmoire pour la source
            ; -----------------------------------------------------------
            invoke CreateCompatibleDC,[ps.hdc]
            mov [.SrcDC],eax

            ; Obtenir la taille du texte  dessiner
            ; -------------------------------------
            mov [rect2.left],0
            mov [rect2.top],0
            mov [rect2.right],0
            mov [rect2.bottom],0
            invoke DrawText,[.SrcDC],Texte,-1,rect2,DT_CALCRECT+\
                                    DT_SINGLELINE+DT_LEFT+DT_TOP

            ; Cration d'un bitmap pour le texte
            ; ----------------------------------
            invoke CreateCompatibleBitmap,[ps.hdc],[rect2.right],[rect2.bottom]
            mov [.hBmp2],eax

            ; L'inclure dans le dispositif de contexte de source
            ; --------------------------------------------------
            invoke SelectObject,[.SrcDC],[.hBmp2]
            push eax

            ; Dessiner le texte
            ; -----------------
            invoke DrawText,[.SrcDC],Texte,-1,rect2,DT_SINGLELINE+DT_LEFT+DT_TOP

            ; Rotation du bitmap
            ; ------------------
            stdcall Rotate,[.hBmp1],[.hBmp2],50,20,0
            stdcall Rotate,[.hBmp1],[.hBmp2],100,20,1
            stdcall Rotate,[.hBmp1],[.hBmp2],150,20,2
            stdcall Rotate,[.hBmp1],[.hBmp2],200,20,3

            ; Slection de l'ancien bitmap
            ; ----------------------------
            pop eax
            invoke SelectObject,[.SrcDC],eax

            ; Supprimer le bitmap source cre
            ; -------------------------------
            invoke DeleteObject,eax

            ; Charger un bitmap depuis une ressource
            ; --------------------------------------
            invoke LoadBitmap,[wc.hInstance],ID_BITMAP
            mov [.hBmp2],eax

            ; L'inclure dans le dispositif de contexte de source
            ; --------------------------------------------------
            invoke SelectObject,[.SrcDC],[.hBmp2]
            push eax

            ; Rotation du bitmap
            ; ------------------
            stdcall Rotate,[.hBmp1],[.hBmp2],50,100,0
            stdcall Rotate,[.hBmp1],[.hBmp2],100,100,1
            stdcall Rotate,[.hBmp1],[.hBmp2],150,100,2
            stdcall Rotate,[.hBmp1],[.hBmp2],200,100,3

            ; Slection de l'ancien bitmap source
            ; -----------------------------------
            pop eax
            invoke SelectObject,[.SrcDC],eax

            ; Supprimer le bitmap source cre
            ; -------------------------------
            invoke DeleteObject,eax

            ; Supprimer le dispositif de contexte de source
            ; ---------------------------------------------
            invoke DeleteDC,[.SrcDC]

            ; Copier le bitmap destination dans la fentre bitmap
            ; ---------------------------------------------------
            invoke BitBlt,[ps.hdc],[rect1.left],[rect1.top],[rect1.right],[rect1.bottom],\
                                                                   [.DestDC],0,0,SRCCOPY

            ; Slection de l'ancien bitmap destination
            ; ----------------------------------------
            pop eax
            invoke SelectObject,[.DestDC],eax

            ; Supprimer le bitmap destination cre
            ; ------------------------------------
            invoke DeleteObject,eax

            ; Supprimer le dispositif de contexte destination
            ; -----------------------------------------------
            invoke DeleteDC,[.DestDC]
            invoke EndPaint,[@hWnd],ps
        .endif

    .elseif [@uMsg] = WM_COMMAND

        mov eax,[@wParam]
        mov edx,eax
        shr edx,16
        and eax,0FFFFh
        .if edx = BN_CLICKED
            .if eax = ID_EXIT
                ; Demande de fin d'application
                ; ----------------------------
                invoke PostQuitMessage,0

            .endif
        .endif

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
proc Rotate uses esi edi, @hBmpDest, @hBmpSrc ,@x ,@y, @nRotate
    local .nbitsd:DWORD, .hmemd:DWORD, .nbitss:DWORD, .hmems:DWORD

    ; Obtenir des infos sur le bitmap destination
    ; -------------------------------------------
    invoke GetObject,[@hBmpDest],sizeof.BITMAP,bmd
    mov eax,[bmd.bmWidthBytes]
    mov edx,[bmd.bmHeight]
    mul edx
    mov [.nbitsd],eax

    ; Allouer de la mmoire pour les bits du bitmap destination
    ; ---------------------------------------------------------
    invoke GlobalAlloc,GMEM_FIXED,[.nbitsd]
    mov [.hmemd],eax

    ; Obtenir des infos sur les bits du bitmap destination
    ; ----------------------------------------------------
    invoke GetBitmapBits,[@hBmpDest],[.nbitsd],[.hmemd]

    ; Obtenir des infos sur le bitmap source
    ; --------------------------------------
    invoke GetObject,[@hBmpSrc],sizeof.BITMAP,bms
    mov eax,[bms.bmWidthBytes]
    mov edx,[bms.bmHeight]
    mul edx
    mov [.nbitss],eax

    ; Allouer de la mmoire pour les bits du bitmap
    ; ---------------------------------------------
    invoke GlobalAlloc,GMEM_FIXED,[.nbitss]
    mov [.hmems],eax

    ; Obtenir les bits source du bitmap
    ; ---------------------------------
    invoke GetBitmapBits,[@hBmpSrc],[.nbitss],[.hmems]

    ; Copier les pixels un par un
    ; ---------------------------
    mov edx,0
    .while edx < [bms.bmHeight]
        mov ecx,0
        .while ecx < [bms.bmWidth]
            push ecx edx
            mov esi,[.hmems]
            push edx
            mov eax,[bms.bmWidthBytes]
            mul edx
            add esi,eax
            movzx eax,[bms.bmBitsPixel]
            shr eax,3
            mul ecx
            add esi,eax
            pop edx
            mov eax,[@nRotate]
            .if eax = 1
                ; Rotation 90 degres
                ; ------------------
                sub edx,[bms.bmHeight]
                neg edx
                xchg ecx,edx
            .elseif eax = 2
                ; Rotation 180 degres
                ; -------------------
                sub edx,[bms.bmHeight]
                neg edx
                sub ecx,[bms.bmWidth]
                neg ecx
            .elseif eax = 3
                ; Rotation 270 degres
                ; -------------------
                sub ecx,[bms.bmWidth]
                neg ecx
                xchg ecx,edx
            .endif

            ; Ajouter les offsets de destination
            ; ----------------------------------
            add ecx,[@x]
            add edx,[@y]
            .if ecx < [bmd.bmWidth] & edx < [bmd.bmHeight]
                ; Calculer l'adresse de destination
                ; ---------------------------------
                mov edi,[.hmemd]
                mov eax,[bmd.bmWidthBytes]
                mul edx
                add edi,eax
                movzx eax,[bmd.bmBitsPixel]
                shr eax,3
                xchg eax,ecx
                mul ecx
                add edi,eax

                ; Copier les octets
                ; -----------------
                rep movsb
            .endif
            pop edx ecx
            inc ecx
        .endw
        inc edx
    .endw

    ; Copier les bits du bitmap destination
    ; -------------------------------------
    invoke SetBitmapBits,[@hBmpDest],[.nbitsd],[.hmemd]

    ; Librer la mmoire alloue
    ; --------------------------
    invoke GlobalFree,[.hmems]
    invoke GlobalFree,[.hmemd]

    ret
endp

;===========================================================================================
proc CreateControls @hWnd

    ; Cration de la fenetre-enfant PUSHBUTTON Exit
    ; ---------------------------------------------
    stdcall Dlg,Button,TxtExit,[@hWnd],224,148,44,20,ID_EXIT,BS_PUSHBUTTON,0

    ret
endp

;===========================================================================================
; Cration d'une fenetre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class    = adresse du nom de control class
; @AddrText = adresse du control text
; @hParent  = handle de la fenetre parent
; @X        = x-coordonne fenetre-enfant
; @Y        = y-coordonne fenetre-enfant
; @wX       = largeur en pixels fenetre-enfant
; @hY       = hauteur en pixels fenetre-enfant
; @Id       = identificateur du controle forme ID_XXX
; @Style    = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle  = window style tendu
;-------------------------------------------------------------------------------------------
; sortie eax = handle de la fenetre-enfant
;-------------------------------------------------------------------------------------------
proc Dlg @Class, @AddrText, @hParent, @X, @Y, @wX, @hY, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AddrText],eax,[@X],[@Y],[@wX],[@hY],\
                          [@hParent],[@Id],[wc.hInstance],NULL
    mov ebx,eax

    ; Valider la fonte standard MS Sans Serif pour le control
    ; -------------------------------------------------------
    invoke SendMessage,ebx,WM_SETFONT,[hFontMSSS],NULL
    mov eax,ebx

    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  Titre       db ' Rotation de bits',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' AsmGges France @2006  ',0

  Texte       db 'Texte',0
  MSSansSerif db 'MS Sans Serif',0

  Button      db 'BUTTON',0
  TxtExit     db 'Exit',0

  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 292
  WinH        dd 212

  hWnd        dd ?
  hFontMSSS   dd ?

  wc          WNDCLASS  0,WindowProc,0,0,NULL,NULL,NULL,COLOR_BTNFACE+1,NULL,ClassName
  msg         MSG
  ps          PAINTSTRUCT
  rect1       RECT
  rect2       RECT
  bmd         BITMAP
  bms         BITMAP

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons,\
            RT_BITMAP,appBmp

  IDI_FRANCE = 17
  ID_BITMAP  = 100
  ID_EXIT    = 200

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  resource appBmp,\
           ID_BITMAP,LANG_NEUTRAL,bmp1

  icon main_icon,icon_data,'res\france.ico'

  bitmap bmp1,'res\as.bmp'

;===========================================================================================
